/****************************************************************************
*** .INI file module of Remote Agent PC 
*** Public include file
*** Copyright 1999 Demmel Hard & Soft

HISTORY:
WHEN			WHO	WHAT
----------------------------------------------------------------------------
1998-10-28	Jerry	Creation
1999-03-29	dh - Adapted for SERIALIO project

****************************************************************************/

#ifndef _INIFILE_
#define _INIFILE_

// Max .INI file line length plus one (i.e. it's 127)
#define MAX_INI_LINE_LEN	128

// Open the .INI file looking for it in the current directory 
// Returns TRUE if open was successful, FALSE otherwise
int ini_open(char *fname);

// Close the .INI file
int ini_close(void);

// Go to the specified section of the .INI file (subsequent ini_readline()s
// will read the lines past the section header
// section_name ... Name of the section to go to (without the brackets)
// Returns TRUE if section was found, FALSE otherwise
int ini_goto_section(char *section_name);

// Return next line after "cooking" (see cooked_readline) but stop at
// a new section
// s ... string the line is to be returned in
// slen ... length of string including \0
// Returns pointer to real begin of line (ignoring whitespace), or 0 if there
// are no more lines
char *ini_section_readline(char *s, unsigned slen);

// Return the current line number of the .INI file
int ini_curr_line(void);

// Return the value of the specified variable in the specified section as a
// string
// section ... name of section in which variable is to be looked for
// varname ... name of variable whose value is to be returned
// s ... pointer to line buffer
// slen ... real length of s
// Returns 0 if section or variable not found
char *ini_get_varstr(char *section, char *varname, char *s, unsigned slen);

// Remove all whitespace from given string
// Returns s.
char *ini_remove_whitespace(char *s);

#endif 	// _INIFILE_
