#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <stdarg.h>
#include <signal.h>

#include "serialio.h"
#include "messages.h"
#include "inifile.h"

#define TITLE_STRING	"Remote Agent PC 880 Serial-Port Demonstration V1.21\n\r"	\
							"Copyright (c) by Demmel, 1999"

#define BIOS_TIMER 0x1A
#define TIMER_READ 0x0

#define SCREEN_WIDTH		80
#define SCREEN_HEIGHT	25

#define WORK_AREA_H		4
#define WORK_AREA_Y		SCREEN_HEIGHT - WORK_AREA_H - 1
#define WORK_AREA_X		0
#define WORK_AREA_W		SCREEN_WIDTH

#define DATA_AREA_Y		3
#define DATA_AREA_X		0

#define MDM_CNTRL_X		37

#define BUFLEN				256
#define RXBUFLEN			SCREEN_WIDTH + 20
#define LEFTMARGIN		8
#define RXDISPLEN			(SCREEN_WIDTH - LEFTMARGIN)

char rx_strings[8][RXBUFLEN] = { 0 };
char rx_string_len[8] = { 0 };

#define ESC			'\x1b'

// flag bits
#define DO_CR			1
#define DO_SPACE		2
#define DECIMAL		4
#define STRING			8

// return vals for getxstring(), gethex() and getdec()
#define GET_ERR		-1
#define GET_OK			0
#define GET_NONE		1

#define MAXADDR		0xfffffL

void scstatus(char *msg);
void show_rx_string(int portno, word c);

// get time stamp in 56 ms units from BIOS
dword time_stamp(void)
{
	union REGS r;

	r.h.ah = TIMER_READ;
	int86(BIOS_TIMER, &r, &r);

	return r.x.dx | (r.x.cx >> 16);
}

// get key routine
int getkey(void)
{
	int err, i;
	word data;

	while (TRUE)
	{
		if (kbhit())
			return getch();
		else
		{
			// scan all open and initialized ports for data

			for (i = 0; i < 8; i++)
			{
				if (sio_IsPortOpen(i, &err) &&
					sio_IsPortInitialized(i, &err))
				{
					if (sio_GetOne(i, &data))
						show_rx_string(i, data);
				}
			}
		}
	}
}

int getyn(void)
{
	int c;

	while (TRUE)
	{
		c = getkey();
		c = toupper(c);

		if (c == 'Y')
		{
			putchar(c);
			return 1;
		}
		
		if (c == 'N')
		{
			putchar(c);
			return -1;
		}

		if (c == ESC)
		{
			scstatus("Last user entry: Input aborted");
			return 0;
		}

		putchar('\a');
	}
}

// misc. string functions for getting input from user:

// get extended string: handles hex, decimal and string inputs
static int getxstring(char *buffer, int len, word flags)
{
	char *p = buffer;
	char *p1 = buffer;
	int c;
	int ret = GET_OK;
	int count = 0;

	while (TRUE)
	{
		c = getkey();

		if (count >= len && !(c == '\b' || c == '\r' || c == '\n' || c == ESC))
		{
			putchar('\a');
			continue;
		}

		if (c == '\b')
		{
			if (--count < 0)
				count = 0;

			p--;

			if (p < p1)
				p = p1;
			else
			{
				printf("\b \b");			// erase char on screen
			}
		}
		else if (c == '\r' || c == '\n')
		{
			if (count == 0 && (flags & STRING) == 0)
				putchar('\a');
			else
			{
				if (flags & DO_SPACE)
					printf("  ");

				if (flags & DO_CR)
					putchar('\n');

				break;
			}
		}
		else if ((c != ESC && (flags & STRING))
			|| (isxdigit(c) && ((flags & (DECIMAL | STRING)) == 0))
			|| (isdigit(c) && (flags & DECIMAL) && (flags & STRING) == 0))
		{
			*p++ = (char) c;
			putchar(c);
			count++;
		}
		else if (c == ESC)
		{
			scstatus("Last user entry: Input aborted");
			ret = GET_ERR;
			break;
		}
		else
			putchar('\a');
	}

	*p = '\0';
	
	if (ret == GET_ERR)
	{
		*buffer = '\0';
		return ret;
	}

	return ret;
}

// convert string to hex
dword atoh(char *s)
{
	dword hex, h;
	int c;

	hex = 0;

	while(*s && *s!='\n' && *s!='\r')
	{
		c = toupper(*s);
		s++;

		if (isxdigit(c))	
		{
			if (isdigit(c))
				h = c - 0x30;
			else
				h = c - 0x37;

			hex <<= 4;
			hex += h;
		}
	}
	return hex;
}

// get a hex value between 0 and 0xffff
int gethex(long *val, word flags)
{
	char	s[10 + 1];
	int ret;

	if ((ret = getxstring(s, 4, flags)) != GET_ERR)
	{
		if (*s == '\0')
			return GET_NONE;

		*val = atoh(s);
		return ret;
	}
	else
		return ret;
}

// get a decimal value between 0 and 99
int getdec(long *val, word flags)
{
	char	s[10 + 1];
	int 	ret;

	if ((ret = getxstring(s, 2, DECIMAL | flags)) != GET_ERR)
	{
		if (*s == '\0')
			return GET_NONE;

		*val = atol(s);
		return ret;
	}
	else
		return ret;
}

// get a decimal value between 0 and 99
int getstring(char *buffer, int count)
{
	int 	ret;

	if ((ret = getxstring(buffer, count, STRING)) != GET_ERR)
	{
		if (*buffer == '\0')
			return GET_NONE;

		return ret;
	}
	else
		return ret;
}

// misc. screen functions

// goto x/y on screen
void scgotoxy(int y, int x)
{
	union REGS r;

	r.h.ah = 0xf;
	int86(0x10, &r, &r);		// get display page in BH

	r.h.ah = 0x2;
	r.h.dh = (unsigned char) y;
	r.h.dl = (unsigned char) x;
	int86(0x10, &r, &r);		// goto required coordinate
}

void scwherexy(int *y, int *x)
{
	union REGS r;

	r.h.ah = 0xf;
	int86(0x10, &r, &r);		// get display page in BH

	r.h.ah = 0x3;
	int86(0x10, &r, &r);		// goto required coordinate

	*y = r.h.dh;
	*x = r.h.dl;
}


// put character c to y/x
void scputc(int c, int y, int x)
{
	union REGS r;

	r.h.ah = 0xf;
	int86(0x10, &r, &r);		// get display page in BH

	r.h.ah = 0x2;
	r.h.dh = (unsigned char) y;
	r.h.dl = (unsigned char) x;
	int86(0x10, &r, &r);		// goto required coordinate

	r.h.ah = 0x9;
	r.h.al = c & 0xff;
	r.h.bl = 0x17 & 0xff;	// hard coded :-) white on blue
	r.h.cl = 1;					// write one times
	r.h.ch = 0;
	int86(0x10, &r, &r);		// now write character c
}

// printf string to y/x
void scprintf(int y, int x, char *fmt,...)
{
	int i, len, c;
	va_list ap;
	char buffer[BUFLEN];

	va_start(ap, fmt);
	vsprintf(buffer, fmt, ap);
	va_end(ap);

	scgotoxy(y, x);

	for (i = 0, len = strlen(buffer); i < len; i++)
	{
		c = buffer[i];

		if (c == '\n')
			y++;
		else if (c == '\r')
			x = 0;
		else 
			scputc(c, y, x++);
	}
}

void scclear(int y, int x, int y1, int x1)
{
	char buffer[BUFLEN];

	if (x1 - x < 1)
		return;
	if (y1 - y < 0)
		return;

	memset(buffer, ' ', x1 - x + 1);
	buffer[x1 - x + 1] = '\0';

	scprintf(y, x, buffer);

	if (y < y1)
	{
		for (++y; y < y1; y++)
			scprintf(y, x, buffer);
	
		scprintf(y1, x, buffer);
	}
}

// clear screen
void sccls(void)
{
	scclear(0, 0, SCREEN_HEIGHT - 1, SCREEN_WIDTH - 1);
}

void scline(int y)
{
	char buffer[SCREEN_WIDTH + 1];

	memset(buffer, '', SCREEN_WIDTH);
	buffer[SCREEN_WIDTH + 1] = '\0';
	scprintf(y, 0, buffer);
}

// show status
void scstatus(char *msg)
{
	// clear old status line
	scclear(SCREEN_HEIGHT - 1, 0, SCREEN_HEIGHT - 1, SCREEN_WIDTH - 1);
	scprintf(SCREEN_HEIGHT - 1, 0, msg);
}

// set cursor to begin of work area
void scgotoworkarea(void)
{
	scgotoxy(WORK_AREA_Y, WORK_AREA_X);
}

// clear work area and set cursor to begin of work area
void scclrworkarea(void)
{
	scclear(WORK_AREA_Y, WORK_AREA_X, WORK_AREA_Y + WORK_AREA_H - 1, WORK_AREA_X + WORK_AREA_W - 1);
	scgotoworkarea();
}

// show function error
void scshowfuncerr(int errno, char *funcname)
{
	scclrworkarea();
	printf("%s returned error # %d:\a\n%s\n",
		funcname, errno, msg_ShowMsg(errno));

	scstatus("Press any key to continue ...");
	getkey();
	scstatus("");
}

// get an address in hex between 0 and 0xffff
int getaddr(long *val, word flags)
{
	int retval;

	if ((retval = gethex(val, flags)) == GET_ERR)
		return GET_ERR;

	if (retval == GET_NONE)
	{
		*val = 0L;
		return GET_NONE;
	}

	if (*val > 0xffff)
	{
		scstatus("Last user entry: Address error");
		putchar('\a');
		return GET_ERR;
	}

	return retval;
}

// get long value (0 ... 999999)
int get_long(long *val, int flags)
{
	char	s[10 + 1];
	int 	ret;

	if ((ret = getxstring(s, 6, DECIMAL | flags)) != GET_ERR)
	{
		if (*s == '\0')
			return GET_NONE;

		*val = atol(s);
		return ret;
	}
	else
		return ret;
}

// get baud rate (300 ... 115200)
int get_baud(long *val, int flags)
{
	int retval;

	printf("Baud rate: ");

	if ((retval = get_long((long *) val, flags)) == GET_ERR)
		return GET_ERR;

	if (retval == GET_NONE)
	{
		*val = 0L;
		return GET_NONE;
	}

	if (*val > 115200 || *val < 300)
	{
		scstatus("Last user entry: Baud rate must be between 300 and 115200");
		putchar('\a');
		return GET_ERR;
	}

	return retval;
}

int getdecrange(char *msg, int min_val, int max_val, int flags)
{
	long val;

	printf(msg);
	if (getdec(&val, flags) != GET_OK)
		return -1;

	if (val < min_val || val > max_val)
	{
		char scratch[100];

		sprintf(scratch, "Last user entry: Value is not between %d and %d\n",
			min_val, max_val);

		scstatus(scratch);
		putchar('\a');
		return -1;
	}

	return (int) val;
}

// get parity
int get_parity(int flags)
{
	int parity;

	if ((parity = getdecrange("Parity (0=none, 1=odd, 2=even): ", 0, 2, flags)) < 0)
		return -1;

	if (parity == 0)
		parity = NONE;
	else if (parity == 1)
		parity = ODD;
	else 
		parity = EVEN;

	return parity;
}

// get # of stop bits
int get_stop(int flags)
{
	int stop;

	if ((stop = getdecrange("Number of stop bits (1 or 2): ", 1, 2, flags)) < 0)
		return -1;

	return stop;
}

// get # of data bits
int get_data(int flags)
{
	int data;

	if ((data = getdecrange("Number of data bits (5 ... 8): ", 5, 8, flags)) < 0)
		return -1;

	return data;
}

// get a port number for serial ports (1 ... 8)
int get_port_no(int flags)
{
	int portno;

	if ((portno = getdecrange("Port number: ", 1, 8, flags)) < 0)
		return -1;

	return (int) portno - 1;
}

// get an interrupt number for serial ports (3 ... 15)
int get_interrupt(int flags)
{
	int intno;

	if ((intno = getdecrange("Interrupt number: ", 3, 15, flags)) < 0)
		return -1;

	return intno;
}

// get an address for serial ports (0x100 ... 0x07ff)
int get_address(int flags)
{
	long address;

	printf("Address (hex): ");
	if (gethex(&address, flags) != GET_OK)
		return -1;

	if (address < (long) 0x100 || address > (long) 0x07ff)
	{
		scstatus("Last user entry: Address must be between 100 and 7ff");
		putchar('\a');
		return -1;
	}

	return (int) address;
}

void getportxy(int portno, int *y, int *x)
{
	*y = DATA_AREA_Y + portno * 2;
	*x = DATA_AREA_X + LEFTMARGIN;
}

void show_port_open(int portno)
{
	int y, x;

	getportxy(portno, &y, &x);
	scprintf(y, x, "Opened");
}

void show_port_closed(int portno)
{
	int y, x;

	getportxy(portno, &y, &x);
	scclear(y, x, y, x + (SCREEN_WIDTH - x - 1));
	scprintf(y, x, "Closed");
}

void show_port_mod_stat(int portno, word status)
{
	int y, x;

	getportxy(portno, &y, &x);
	// " DTR:? RTS:? CTS:? DSR:? RI:? CD:?"
	scprintf(y, x+ MDM_CNTRL_X + 18, "%d", (status & CTS) ? 1 : 0);
	scprintf(y, x+ MDM_CNTRL_X + 24, "%d", (status & DSR) ? 1 : 0);
	scprintf(y, x+ MDM_CNTRL_X + 29, "%d", (status & RI) ? 1 : 0);
	scprintf(y, x+ MDM_CNTRL_X + 34, "%d", (status & CD) ? 1 : 0);
}

void show_port_rts_dtr(int portno, int dtr, int rts)
{
	int y, x;

	getportxy(portno, &y, &x);
	// " DTR:? RTS:? CTS:? DSR:? RI:? CD:?"
	scprintf(y, x+ MDM_CNTRL_X + 6, "%d", dtr ? 1 : 0);
	scprintf(y, x+ MDM_CNTRL_X + 12, "%d", rts ? 1 : 0);
}

void show_port_init(int portno, long baud, int parity, int stop, int bit)
{
	int y, x;

	getportxy(portno, &y, &x);
	scclear(y, x, y, x + MDM_CNTRL_X - 1);
	scprintf(y, x, "bps:%ld data:%d stop:%d parity: ", baud, bit, stop);

	switch (parity)
	{
		case NONE:
			printf("no");
			break;
		case EVEN:
			printf("even");
			break;
		case ODD:
			printf("odd");
			break;
	}

	scprintf(y, x + MDM_CNTRL_X, " DTR:? RTS:? CTS:? DSR:? RI:? CD:?");
	show_port_rts_dtr(portno, TRUE, TRUE);

	show_port_mod_stat(portno, sio_GetModemStatus(portno));
}

// interpret word got from sio_GetOne() or sio_GetBuffer()
int interpret_buff_char(word c, char *s, int *status)
{
	word flags;

	*status = -1;

	if ((c & 0xff00) == 0)
	{
		// normal character ?
		if (isprint(c))
		{
			*s++ = (char) c;
			*s = '\0';
			return 1;
		}
		else
		{
			// special character
			sprintf(s, "[%02X]", c);
			return 4;
		}
	}
	else if (c & (STAT_CHNG << 8))
	{
		// modem status lines have changed

		*status = (c & 0xff);
		
		sprintf(s, "<S:%c%c%c%c>",
			(c & CTS) ? 'C' : 'c',
			(c & DSR) ? 'D' : 'd',
			(c & RI) ? 'R' : 'r',
			(c & CD) ? 'A' : 'a');

		return 8;
	}

	// we had a receive error, flags are located in highbyte
	flags = (c >> 8) & 0xff;

	sprintf(s, "<E:%02X/%c%c%c%c>", c & 0xff, 
		(flags & OE) ? 'O' : 'o',
		(flags & PE) ? 'P' : 'p',
		(flags & FE) ? 'F' : 'f',
		(flags & BI) ? 'B' : 'b');

	return 11;
}

// interpret word got from sio_GetOne() or sio_GetBuffer() and compile string
void show_rx_string(int portno, word c)
{
	int len, newlen, y, x, old_y, old_x;
	int status;

	len = interpret_buff_char(c, &rx_strings[portno][rx_string_len[portno]],
			&status);

	scwherexy(&old_y, &old_x);

	if (status >= 0)
		show_port_mod_stat(portno, status);

	newlen = rx_string_len[portno] + len;

	if (newlen > RXDISPLEN)
	{
		memmove(rx_strings[portno], rx_strings[portno] + newlen - RXDISPLEN,
			RXDISPLEN + 1);

		rx_string_len[portno] = RXDISPLEN;

		// we must print the whole string
		getportxy(portno, &y, &x);
		scprintf(y + 1, x, rx_strings[portno]);
	}
	else
	{
		// we can print a partial string

		getportxy(portno, &y, &x);
		scprintf(y + 1, x + rx_string_len[portno], rx_strings[portno] +
			rx_string_len[portno]);

		rx_string_len[portno] = newlen;
	}

	scgotoxy(old_y, old_x);
}

int clr_rx_string(void)
{
	int portno, i, y, x;

	scstatus("Enter parameters for wiping RX data string(s)");
	scclrworkarea();

	if ((portno = getdecrange("Wipe RX string(s) for port number (0 = all, 1 ... 8): ", 0, 8, DO_CR)) < 0)
		return FALSE;

	portno--;

	for (i = 0; i < 8; i++)
	{
		if (i == portno || portno < 0)
		{
			rx_strings[i][0] = '\0';
			rx_string_len[i] = 0;

			getportxy(i, &y, &x);
			scclear(y + 1, x, y + 1, x + RXDISPLEN - 1);
		}
	}

	return TRUE;
}

void screenmask(void)
{
	int i;
	int y = DATA_AREA_Y;

	for (i = 0; i < 8; i++)
	{
		scprintf(y++, DATA_AREA_X, "Port %d: Closed", i + 1);
		scprintf(y++, DATA_AREA_X, "--> RX: ", i + 1);
	}
}

// port related functions:

// open standard COM port
int open_com(void)
{
	int portno, address, intno, err;

	scstatus("Enter COM parameters for opening COM port");
	scclrworkarea();

	if ((portno = get_port_no(DO_SPACE)) < 0)
		return FALSE;
	if ((address = get_address(DO_SPACE)) < 0)
		return FALSE;
	if ((intno = get_interrupt(DO_CR)) < 0)
		return FALSE;

	if ((err = sio_OpenCOMPort(portno, address, intno)) != 0)
	{
		scshowfuncerr(err, "sio_OpenCOMPort()");
		return FALSE;
	}

	show_port_open(portno);
	scstatus("Port successfully opened");
	return TRUE;
}

// open digiboard ports
int open_digiboard(void)
{
	int portstart, portcount, intno, err, i, open;
	long start_address, isr_address;

	open = sio_IsDigiboardOpen(0, &err);
	if (err)
	{
		scshowfuncerr(err, "sio_OpenCOMPort()");
		return FALSE;
	}

	if (open)
	{
		scstatus("Last user entry: Digiboard #0 is already open");
		putchar('\a');
		return FALSE;
	}

	scstatus("Enter COM parameters for opening digiboard #0");
	scclrworkarea();

	if ((portstart = getdecrange("Port start number (3 ... 8): ", 3, 8, DO_SPACE)) < 0)
		return FALSE;

	if ((portcount = getdecrange("Number of ports to open (1 ... 6): ", 1, 6, DO_CR)) < 0)
		return FALSE;

	if (portstart + portcount > 8 + 1)
	{
		scstatus("Last user entry: Range of ports to open exceeds port # 8");
		putchar('\a');
		return FALSE;
	}

	printf("Start address of 1st port (hex): ");
	if (gethex(&start_address, DO_SPACE) != GET_OK)
		return FALSE;

	if (start_address < (long) 0x100 || start_address > (long) 0x07ff)
	{
		scstatus("Last user entry: Start address must be between 100 and 7ff");
		putchar('\a');
		return FALSE;
	}

	printf("Address of IR status register (hex): ");
	if (gethex(&isr_address, DO_CR) != GET_OK)
		return FALSE;

	if (isr_address < (long) 0x100 || isr_address > (long) 0x07ff)
	{
		scstatus("Error: Interrupt status register address must be between 100 and 7ff");
		putchar('\a');
		return FALSE;
	}

	if ((intno = get_interrupt(DO_CR)) < 0)
		return FALSE;

	portstart--;
	if ((err = sio_OpenDigiboard(0, portstart, portcount,
		(word) start_address, (word) isr_address, intno)) != 0)
	{
		scshowfuncerr(err, "sio_OpenDigiboard()");
		return FALSE;
	}

	for (i = 0; i < portcount; i++)
		show_port_open(portstart++);

	scstatus("Digiboard successfully opened");
	return TRUE;
}

// open digiboard ports
int close_digiboard(void)
{
	int portstart, portcount, err, i, open;

	open = sio_IsDigiboardOpen(0, &err);
	if (err)
	{
		scshowfuncerr(err, "sio_IsDigiboardOpen()");
		return FALSE;
	}

	if (!open)
	{
		scstatus("Last user entry: Digiboard #0 is not open");
		putchar('\a');
		return FALSE;
	}

	err = sio_GetDigiboardInfo(0, &portstart, &portcount);
	if (err)
	{
		scshowfuncerr(err, "sio_GetDigiboardInfo()");
		return FALSE;
	}

	if ((err = sio_CloseDigiboard(0)) != 0)
	{
		scshowfuncerr(err, "sio_CloseDigiboard()");
		return FALSE;
	}

	for (i = 0; i < portcount; i++)
		show_port_closed(portstart++);

	scstatus("Digiboard successfully closed");
	return TRUE;
}

// close standard COM port
int close_com(void)
{
	int portno, err;

	scstatus("Enter COM parameters for closing COM port");
	scclrworkarea();

	if ((portno = get_port_no(DO_CR)) < 0)
		return FALSE;

	if ((err = sio_CloseCOMPort(portno)) != 0)
	{
		scshowfuncerr(err, "sio_CloseCOMPort()");
		return FALSE;
	}

	show_port_closed(portno);
	scstatus("Port successfully closed");
	return TRUE;
}

int check_open(int portno, char *msg)
{
	int open, err;

	open = sio_IsPortOpen(portno, &err);

	if (err)
	{
		scshowfuncerr(err, "sio_IsPortOpen()");
		return FALSE;
	}

	if (!open)
	{
		scclrworkarea();
		printf("The requested port # %d is not opened.\a\n\r"
			"%s", portno + 1, msg);

		scstatus("Press any key to continue ...");
		getkey();
		scstatus("");

		return FALSE;
	}

	return TRUE;
}

int check_init(int portno, char *msg)
{
	int init, err;

	init = sio_IsPortInitialized(portno, &err);

	if (err)
	{
		scshowfuncerr(err, "sio_IsPortInitialized()");
		return FALSE;
	}

	if (!init)
	{
		scclrworkarea();
		printf("The requested port # %d is not initialized.\a\n\r"
			"%s", portno + 1, msg);

		scstatus("Press any key to continue ...");
		getkey();
		scstatus("");

		return FALSE;
	}

	return TRUE;
}

int init_port(void)
{
	int portno, err, parity, stop, databits;
	long baudrate;

	scstatus("Enter COM parameters for initializing COM port");
	scclrworkarea();

	if ((portno = get_port_no(DO_SPACE)) < 0)
		return FALSE;

	if (!check_open(portno, "Please open the COM port before initializing it."))
		return FALSE;

	if (get_baud(&baudrate, DO_SPACE) != GET_OK)
		return FALSE;

	if ((parity = get_parity(DO_CR)) < 0)
		return FALSE;

	if ((stop = get_stop(DO_SPACE)) < 0)
		return FALSE;

	if ((databits = get_data(DO_SPACE)) < 0)
		return FALSE;

	if ((err = sio_InitPort(portno, baudrate, parity, stop, databits,
		RX_AVAIL | MODEM_STAT | LINE_STAT)) != 0)
	{
		scshowfuncerr(err, "sio_InitPort()");
		return FALSE;
	}

	show_port_init(portno, baudrate, parity, stop, databits);
	scstatus("Port successfully initialized");
	return TRUE;
}

int modem_control(void)
{
	int portno, rts, dtr;

	scstatus("Set modem control lines");
	scclrworkarea();

	if ((portno = get_port_no(DO_CR)) < 0)
		return FALSE;

	if (!check_open(portno, "Please open the COM port before setting modem control lines."))
		return FALSE;

	if (!check_init(portno, "Please initialize the COM port before trying to set modem control lines."))
		return FALSE;

	if ((dtr = getdecrange("Set DTR bit (0/1): ", 0, 1, DO_SPACE)) < 0)
		return FALSE;

	if ((rts = getdecrange("Set RTS bit (0/1): ", 0, 1, DO_SPACE)) < 0)
		return FALSE;

	sio_SetModemControl(portno, (dtr ? DTR : 0) | (rts ? RTS : 0));
	show_port_rts_dtr(portno, dtr, rts);
	scstatus("Modem control lines successfully set");

	return TRUE;
}

char *convert_string(char *instring, char *outstring, int *len)
{
	char *i = instring;
	char *i1;
	char *o = outstring;
	int num, count, mult;
	byte chksum = 0;

	while (*i)
	{
		switch (*i)
		{
			case '\\':
				i1 = i;

				if (toupper(*(i + 1)) == 'X')
				{
					num = 0;
					i += 2;
					mult = 0x10;

					if (!isxdigit(*i))
					{
						i = i1;			// roll back
						*o = *i++;
						chksum += *o++;

						break;
					}
					else
					{
						for (count = 0; count < 2 && *i != 0; count++)
						{
							if (isxdigit(*i))
							{
								*i = toupper(*i);
								num += ((*i > '9') ? (*i - 'A' + 0x0a) : (*i - '0')) *
									mult;
								i++;
								mult /= 0x10;
							}
							else
								break;
						}
						if (mult != 0)
							num /= 0x10;
					}
				}
				else if (*(i + 1) == 'r')
				{
					*o = '\r';
					chksum += *o++;
					i += 2;
					break;
				}
				else if (*(i + 1) == 'n')
				{
					*o = '\n';
					chksum += *o++;
					i += 2;
					break;
				}
				else if (*(i + 1) == 't')
				{
					*o = '\t';
					chksum += *o++;
					i += 2;
					break;
				}
				else if (*(i + 1) == 'f')
				{
					*o = '\f';
					chksum += *o++;
					i += 2;
					break;
				}
				else if (*(i + 1) == 'c')
				{
					chksum = (byte) ((word) 0x100 - (word) (chksum & 0xff));
					*o++ = chksum;
					i += 2;
					chksum = 0;
					break;
				}		
				else
					continue;

				if (num > 255)
				{
					i = i1;			// roll back
					*o = *i++;
					chksum += *o++;
					break;
				}

				*o = (unsigned char) num;
				chksum += *o++;
				break;

			default:
				*o = *i++;
				chksum += *o++;
				break;
		}
	}
	*o = '\0';
	*len = o - outstring;

	return o;
}

int do_transmit(void)
{
	int send_cr, portno;
	char s[BUFLEN];
	char so[BUFLEN];
	int len, old_y, old_x;

	scstatus("Enter data to transmit");
	scclrworkarea();

	if ((portno = get_port_no(DO_SPACE)) < 0)
		return FALSE;

	if (!check_open(portno, "Please open the COM port before trying to transmit data."))
		return FALSE;

	if (!check_init(portno, "Please initialize the COM port before trying to transmit data."))
		return FALSE;

	printf("Send trailing CR [Y/N]: ");
	if ((send_cr = getyn()) == 0)
		return FALSE;

	scwherexy(&old_y, &old_x);
	scstatus("Hint: Text can contain \\xHH (HH being a hex number), \\t, \\f, \\n and \\r");
	scgotoxy(old_y, old_x);

	printf("\nEnter text: ");
	if (getstring(s, SCREEN_WIDTH - strlen("Enter text: ") - 1) == GET_ERR)
		return FALSE;

	if (send_cr > 0)
	{
		int len = strlen(s);

		s[len] = '\r';
		s[len + 1] = '\0';
	}

	convert_string(s, so, &len);
	sio_SendBuffer(portno, so, len);

	scstatus("String successfully sent");
	return TRUE;
}

int break_out(void)
{
	int portno;

	scstatus("Select port to transmit a break");
	scclrworkarea();

	if ((portno = get_port_no(DO_SPACE)) < 0)
		return FALSE;

	if (!check_open(portno, "Please open the COM port before trying to transmit data."))
		return FALSE;

	if (!check_init(portno, "Please initialize the COM port before trying to transmit data."))
		return FALSE;

	scclrworkarea();
	printf("Now transmit line of port %d is in break state.\n"
		"Hit any key to stop break ...", portno + 1);

	sio_BreakOn(portno);
	getkey();
	sio_BreakOff(portno);

	scstatus("");
	return TRUE;
}

int close_all(void)
{
	int err, i;

	scclrworkarea();
	printf("Close all ports:\n"
		"Are you sure you want to do this (Y/N) : ");

	if (getyn() <= 0)
		return FALSE;

	if ((err = sio_CloseAllPorts()) != 0)
	{
		scshowfuncerr(err, "sio_CloseAllPorts()");
		return FALSE;
	}
	else
	{
		for (i = 0; i < 8; i++)
			show_port_closed(i);
	}

	scstatus("");
	return TRUE;
}

int setup_all(int ask)
{
	char section[30];
	char buffer[MAX_INI_LINE_LEN + 1];
	char *val;
	int portstart = 0, portcount = 0, intno = 0, err = FALSE, i;
	long start_address = 0, isr_address = 0;
	int parity, stopbits, databits, open;
	long baudrate;

	scclrworkarea();
	if (ask)
	{
		printf("Open and initialize all ports with values found in SERIALIO.INI\n"
			"Are you sure you want to do this (Y/N) : ");

		if (getyn() <= 0)
			return FALSE;
	}

	if (ini_open("SERIALIO.INI"))
	{
		scclrworkarea();
		printf("\nReading and executing data of SERIALIO.INI ...");

		for (i = 0; i < 4; i++)
		{
			start_address = 0;
			intno = 0;

			sprintf(section, "Com%d", i + 1);

			if ((val = ini_get_varstr(section, "PortBaseAddr", buffer,
				MAX_INI_LINE_LEN)) != NULL)
			{
				start_address = atoh(val);

				if ((val = ini_get_varstr(section, "Interrupt", buffer,
					MAX_INI_LINE_LEN)) != NULL)
				{
					intno = atoi(val);
				}

				if (start_address == 0 || intno == 0)
				{
					scclrworkarea();
					printf("Invalid/incomplete data in SERIALIO.INI in section [%s]\n"
						"Press any key to continue ...\a", section);
					getkey();
					printf("\nReading and executing data in SERIALIO.INI ...");
				}
				else
				{
					if ((err = sio_OpenCOMPort(i, (word) start_address, intno)) != 0)
					{
						scshowfuncerr(err, "sio_OpenCOMPort()");
						return FALSE;
					}

					show_port_open(i);
				}
			}
		}

		strcpy(section, "Digiboard");

		strcpy(section, "Digiboard");
		if ((val = ini_get_varstr(section, "FirstPort", buffer,
			MAX_INI_LINE_LEN)) != NULL)
		{
		 	portstart = atoi(val) - 1;

			if ((val = ini_get_varstr(section, "NoOfPorts", buffer,
				MAX_INI_LINE_LEN)) != NULL)
			{
				portcount = atoi(val);

				if ((val = ini_get_varstr(section, "PortBaseAddr", buffer,
					MAX_INI_LINE_LEN)) != NULL)
				{
					start_address = atoh(val);

					if ((val = ini_get_varstr(section, "StatusRegAddr", buffer,
						MAX_INI_LINE_LEN)) != NULL)
					{
						isr_address = atoh(val);

						if ((val = ini_get_varstr(section, "Interrupt", buffer,
							MAX_INI_LINE_LEN)) != NULL)
						{
							intno = atoi(val);
						}
					}
				}
			}

			// now we got the Digiboard stuff from the INI file
			// do a raw check if paramters only

			if (portstart == 0 || portcount == 0 || intno == 0 ||
				start_address == 0 || isr_address == 0)
			{
				scclrworkarea();
				printf("Invalid/incomplete data in SERIALIO.INI in section [%s]\n"
					"Press any key to continue ...\a", section);
				getkey();
				printf("\nReading and executing data in SERIALIO.INI ...");
			}
			else
			{
				if ((err = sio_OpenDigiboard(0, portstart, portcount,
					(word) start_address, (word) isr_address, intno)) != 0)
				{
					scshowfuncerr(err, "sio_OpenDigiboard()");
					return FALSE;
				}
				else
				{
					for (i = 0; i < portcount; i++)
						show_port_open(portstart++);
				}
			}
		}

		for (i = 0; i < 8; i++)
		{
			open = sio_IsPortOpen(i, &err);

			if (err)
			{
				scshowfuncerr(err, "sio_IsPortOpen()");
				return FALSE;
			}

			if (open)
			{
				// port is open - do init now

				sprintf(section, "Port%d", i + 1);
				parity = -1;
				baudrate = 0;
				stopbits = -1;
				databits = 0;

				if ((val = ini_get_varstr(section, "Baudrate", buffer,
					MAX_INI_LINE_LEN)) != NULL)
				{
		 			baudrate = atol(val);

					if ((val = ini_get_varstr(section, "Parity", buffer,
						MAX_INI_LINE_LEN)) != NULL)
					{
						if (stricmp(val, "No") == 0)
							parity = NONE;
						else if (stricmp(val, "Even") == 0)
							parity = EVEN;
						else if (stricmp(val, "Odd") == 0)
							parity = ODD;
						else
							parity = -1;

						if ((val = ini_get_varstr(section, "Stopbits", buffer,
							MAX_INI_LINE_LEN)) != NULL)
						{
							stopbits = atoi(val);

							if ((val = ini_get_varstr(section, "Databits", buffer,
								MAX_INI_LINE_LEN)) != NULL)
							{
								databits = atoi(val);

								if (databits == 0 || parity < 0 || stopbits < 0
									|| baudrate == 0)
								{
									scclrworkarea();
									printf("Invalid/incomplete data in SERIALIO.INI in section [%s]\n"
										"Press any key to continue ...\a", section);
									getkey();
									printf("\nReading and executing data in SERIALIO.INI ...");
								}
								else
								{
									// init port
									if ((err = sio_InitPort(i, baudrate, parity,
										stopbits, databits, RX_AVAIL | MODEM_STAT |
										LINE_STAT)) != 0)
									{
										scshowfuncerr(err, "sio_InitPort()");
										return FALSE;
									}
									else
									{
										show_port_init(i, baudrate, parity, stopbits,
											databits);
									}
								}
							}
						}
					}
				}
			}
		}
		ini_close();
	}
	else
	{
		scstatus("Could not initialize: Could not open SERIALIO.INI");
		putchar('\a');
		return FALSE;
	}

	scstatus("");
	return TRUE;
}

int dispatch(int key)
{
	if (key == '\n' || key == '\r' || key == ESC)
		key = ' ';

	printf("%c", key);

	switch (key)
	{
		case 'O':			// open COM port
			open_com();
			return FALSE;

		case 'A':
			setup_all(TRUE);
			return FALSE;

		case 'B':
			break_out();
			return FALSE;

		case 'S':
			close_all();
			return FALSE;

		case 'D':			// open digiboard
			open_digiboard();
			return FALSE;

		case 'C':			// close COM port
			close_com();
			return FALSE;

		case 'I':			// init COM port
			init_port();
			return FALSE;

		case 'L':			// close Digiboard
			close_digiboard();
			return FALSE;

		case 'T':			// transmit something
			do_transmit();
			return FALSE;

		case 'M':
			modem_control();
			return FALSE;

		case 'W':			//	clear RX strings
			clr_rx_string();
			return FALSE;

		case 'X':			// exit
			return TRUE;

		default:
			printf("\b \b\a");
			break;
	}

	return FALSE;
}

void control_c(int sig)
{
	sig = sig;		// satisfy compiler

	signal(SIGINT, control_c);
}

int main(void)
{
	int c;
	int err, i;
	char buffer[MAX_INI_LINE_LEN + 1];
	char *val;

	sccls();
	scprintf(0, 0, TITLE_STRING);
	scline(DATA_AREA_Y - 1);
	screenmask();
	scline(WORK_AREA_Y - 1);

	signal(SIGINT, control_c);

	if (ini_open("SERIALIO.INI"))
	{
		if ((val = ini_get_varstr("General", "AutoInit", buffer,
			MAX_INI_LINE_LEN)) != NULL)
		{
			if (atoi(val) > 0)
			{
				setup_all(FALSE);
			}
		}
		ini_close();
	}

	do
	{
		scclrworkarea();
		scprintf(WORK_AREA_Y, WORK_AREA_X,
			"Enter command key: ");
		scprintf(WORK_AREA_Y + 1, WORK_AREA_X,
			"Open/Init <A>ll, <O>pen COM, <C>lose COM, Open <D>igiboard, C<l>ose Digiboard");
		scprintf(WORK_AREA_Y + 2, WORK_AREA_X,
			"<I>nit port, <T>ransmit, <B>reak, <W>ipe RX, <M>odem Cntrl, Clo<s>e All, E<x>it");

		scgotoxy(WORK_AREA_Y, WORK_AREA_X + strlen("Enter command key: "));

		c = getkey();
		c = toupper(c);
	}
	while (!dispatch(c));

	if ((err = sio_CloseAllPorts()) != 0)
		scshowfuncerr(err, "sio_CloseAllPorts()");

	for (i = 0; i < 8; i++)
		show_port_closed(i);

	scstatus("");
	scgotoxy(SCREEN_HEIGHT - 2, 0);

	return 0;
}
