#ifndef _PORTS_
#define _PORTS_

// offset for UART registers
#define _TXREG 	0		// transmit reg */
#define _RXREG 	0		// receive register
#define _DIVLL		0		// divisor latch register low
#define _DIVLH		1		// divisor latch register high
#define _INT_EN	1		// interrupt enable register
#define _INTID 	2		// interrupt ID register 
#define _FIFOCONT 2		// FIFO control register
#define _LINCONT	3		// line control register 
#define _MODCONT	4		// modem control register 
#define _LINSTAT	5		// line status register
#define _MODSTAT	6		// modem status register
#define _SCRATCH	7		// scratch register

#define TXREG(b) 		(b + _TXREG)
#define RXREG(b)		(b + _RXREG)
#define DIVLL(b)		(b + _DIVLL)
#define DIVLH(b)		(b + _DIVLH)
#define INT_EN(b)		(b + _INT_EN)
#define INTID(b)		(b + _INTID)
#define FIFOCONT(b)	(b + _FIFOCONT)
#define LINCONT(b)	(b + _LINCONT)
#define MODCONT(b)	(b + _MODCONT)
#define LINSTAT(b)	(b + _LINSTAT)
#define MODSTAT(b)	(b + _MODSTAT)
#define SCRATCH(b)	(b + _SCRATCH)
  
// parity definements
  
#define NONE 0
#define ODD 1
#define EVEN 2
  
// bit definements for line control register 
#define WLENGTH_L		0x01
#define WLENGTH_M		0x02
#define STOPBIT2		0x04
#define PENABLE		0x08
#define PSELECT		0x10
#define PONE			0x20
#define BREAKOUT		0x40
#define DLAB			0x80

// bit definements for FIFO control register 
#define F_EN		0x01
#define F_RX_RES	0x02
#define F_TX_RES	0x02
#define F_DMA		0x04
#define F_RX_1		0x00
#define F_RX_4		0x40
#define F_RX_8		0x80
#define F_RX_14	0xc0

// bit definements for line status register 
#define DR     	0x01
#define OE     	0x02
#define PE     	0x04
#define FE     	0x08
#define BI     	0x10
#define THRE   	0x20
#define TSRE   	0x40
// this is NOT a bit in the line status register, it's only used to indicate
// that the modem status bits have changed
#define STAT_CHNG 0x80 	

#define HARD_ERROR   (OE | PE | FE | BI)

/* interrupt definements */

#define PIC_ADDR			0x20
#define PIC2_ADDR			0xA0
#define PIC_EOI			0x20

// interrupt mask
#define PIC_MASK			0x21
#define PIC2_MASK			0xA1

// modem pins definements 
  
#define DTR 0x1                  /* on UART for out */
#define RTS 0x2
#define OUT1 0x4
#define OUT2 0x8

// modem status bits
#define CTS		0x10
#define DSR		0x20
#define RI		0x40
#define CD		0x80

// interrupt bits to be enabled in INT_EN
#define RX_AVAIL		0x01
#define TX_EMPTY 		0x02
#define LINE_STAT 	0x04
#define MODEM_STAT	0x08

// interrupt id register values
#define IID_LINE_STAT	0x06
#define IID_REC_DATA		0x04
#define IID_TX_EMPTY		0x02
#define IID_MODEM_STAT	0x00
#define IID_NO_INTR		0x01
#define IID_TIMEOUT		0x0c		// 16550 only

#endif		// _PORTS_
