#include <stdio.h>
#include <stdlib.h>

#include "queue.h"

int queue_Init(queue_t *q, word size_wanted)
{
	word i;

	// check if queue was already initialized
	if (q->data != NULL)
		return QERR_ALR_INIT;

	if (size_wanted < MIN_QUEUE_SIZE || size_wanted > MAX_QUEUE_SIZE)
		return QERR_ILL_SIZE;

	q->mask = q->head = q->tail = q->size = q->tot_size = 0;

	// check if size wanted is a power of 2 value (e.g. 16, 32, 64, ...)
	for (i = 2, q->mask = 0; i <= MAX_QUEUE_SIZE; i *= 2)
	{
		if (i == size_wanted)
		{
			q->mask = i - 1;
			q->tot_size = i;
			break;
		}
	}

	if (q->mask == 0)
		return QERR_ILL_SIZE;	// not a power of 2

	if ((q->data = malloc(size_wanted * sizeof(short))) == NULL)
		return QERR_NO_MEM;

	return QERR_NO_ERR;
}

int queue_DeInit(queue_t *q)
{
	if (q->data == NULL)
		return QERR_DEINIT;

	free(q->data);

	q->data = NULL;
	q->mask = q->head = q->tail = q->size = q->tot_size = 0;

	return QERR_NO_ERR;
}

