#ifndef _QUEUE_
#define _QUEUE_

#include "global.h"

typedef struct
{
	word *data;
	word head;
	word tail;
	word size;
	word tot_size;
	word mask;
} queue_t;

typedef enum
{
	QERR_NO_ERR = 0,		// no error
	QERR_ILL_SIZE = 10,	// queue size is not a power of 2, too small or large
	QERR_NO_MEM,			// no memory for allocating space for queue
	QERR_ALR_INIT,			// queue is already initialized
	QERR_DEINIT				// error on de-initializing queue
} queue_err_t;

#define MIN_QUEUE_SIZE	16				// min. size is 16 
#define DEF_QUEUE_SIZE	1024			// default size
#define MAX_QUEUE_SIZE	0x8000		// max. size is 32k

#define queue_ResetQueue(queue)			(queue##)->head = (queue##)->tail = 0;		\
													(queue##)->size = 0
#define queue_Exists(q)						(q##)->data != NULL

#define queue_IsQueueEmpty(queue)		(! (queue##)->size)
#define queue_IsQueueNotEmpty(queue)	((queue##)->size)
#define queue_WaitForQueueSpace(queue) while ((queue##)->size == (queue##)->tot_size) \
														;
#define queue_IsSpaceInQueue(queue)		((queue##)->size < (queue##)->tot_size)

#define queue_FillQueue(q,v)				(q##)->data[(q##)->mask & (q##)->head] = v; \
													(q##)->head++;											\
													(q##)->size++

#define queue_ReadQueue(q,b)				b = (q##)->data[(q##)->mask & (q##)->tail]; \
													(q##)->tail++;											\
													(q##)->size--

#define queue_IsQueueLowWater(q)			((q##)->size < ((q##)->tot_size / 3))
#define queue_IsTxQueueTwoThirdFull(q)	((q##)->size < ((q##)->tot_size / 3 * 2))
#define queue_IsQueueHighWater(q)		((q##)->size > ((q##)->tot_size / 2))

int queue_Init(queue_t *q, word size_wanted);
int queue_DeInit(queue_t *q);

#endif	// _QUEUE_
