#ifndef _SERIALIO_
#define _SERIALIO_

#include "compiler.h"
#include "global.h"
#include "ports.h"
#include "queue.h"

// this are the default port base addressed used
#define COM1_BASE			0x3f8
#define COM2_BASE			0x2f8
#define COM3_BASE			0x3e8
#define COM4_BASE			0x2e8
#define DIGIBOARD_BASE	0x100
#define DIGIBOARD_REG	0x140

#define COM1	0
#define COM2	1
#define COM3	2
#define COM4	3
#define COM5	4
#define COM6	5
#define COM7	6
#define COM8	7
#define COM9	8
#define COM10	9
#define COM11	10
#define COM12	11
#define COM13	12
#define COM14	13
#define COM15	14
#define COM16	15

typedef enum
{
	SIO_NO_ERR = 0,		// no error
	SIO_ALR_HOOKED = 20,	// ISR already hooked
	SIO_INVAL_INT,		  	// invalid interrupt number
	SIO_ALR_UNHOOKED,		// ISR already unhooked
	SIO_INVAL_PORTNO,		// port_no, first_port_no or no_of_ports
								// is invalid
	SIO_INVAL_DB_NO, 		// invalid Digiboard number
	SIO_PORT_USE,			// port already in use
	SIO_INVAL_ADDR,		// invalid address specified
	SIO_PORT_NOPEN,		// port not open
	SIO_DIGI_NOPEN,		// Digiboard not open
	SIO_NOT_STD_COM,		// Port belongs to a digiboard group, close with appropriate function
	SIO_PORT_NOT_INIT,	// port not initialized
	SIO_DIGI_ALR_OPEN,	// digiboard is already open
	SIO_NO_PORT				// Serial port does not exist
} sio_err_t;

#define sio_TXREG(PortNo) 		TXREG(sio_BAddr(PortNo))
#define sio_RXREG(PortNo)		RXREG(sio_BAddr(PortNo))
#define sio_DIVLL(PortNo)		DIVLL(sio_BAddr(PortNo))
#define sio_DIVLH(PortNo)		DIVLH(sio_BAddr(PortNo))
#define sio_INT_EN(PortNo)		INT_EN(sio_BAddr(PortNo))
#define sio_INTID(PortNo)		INTID(sio_BAddr(PortNo))
#define sio_FIFOCONT(PortNo)	FIFOCONT(sio_BAddr(PortNo))
#define sio_LINCONT(PortNo)	LINCONT(sio_BAddr(PortNo))
#define sio_MODCONT(PortNo)	MODCONT(sio_BAddr(PortNo))
#define sio_LINSTAT(PortNo)	LINSTAT(sio_BAddr(PortNo))
#define sio_MODSTAT(PortNo)	MODSTAT(sio_BAddr(PortNo))
#define sio_SCRATCH(PortNo)	SCRATCH(sio_BAddr(PortNo))

int sio_OpenDigiboard(int digiboard_no, int first_port_no, int no_of_ports,
		word first_port_addr, word status_reg_addr, int interrupt_no);
int sio_IsDigiboardOpen(int digiboard_no, int *errno);
int sio_CloseDigiboard(int digiboard_no);
int sio_GetDigiboardInfo(int digiboard_no, int *first_port, int *no_of_ports);
int sio_OpenCOMPort(int port_no, word port_addr, int interrupt_no);
int sio_IsPortOpen(int portno, int *errno);
int sio_CloseCOMPort(int port_no);
int sio_CloseAllPorts(void);
int sio_InitPort(int portno, dword baudrate, int parity,
					int stop, int databits, word interrupt_mask);
int sio_IsPortInitialized(int portno, int *errno);
int sio_ClearRXQueue(int portno);
void sio_SendChar(int portno, int c);
void sio_SendString(int portno, char *s);
void sio_SendBuffer(int portno, byte *s, int len);
int sio_GetOne(int portno, word *s);
int sio_GetBuffer(int portno, word *s, word maxsize);
word sio_GetModemStatus(int portno);
void sio_SetModemControl(int portno, word value);
void sio_BreakOn(int portno);
void sio_BreakOff(int portno);

#endif // _SERIALIO_
